@@ -4,7 +4,7 @@ djsupervisor:  easy integration between django and supervisord
 ==============================================================
 
 
-Django-supervisor combines the process-managment awesomness of supervisord
+Django-supervisor combines the process-management awesomeness of supervisord
 with the convenience of Django's management scripts.
 
 
@@ -189,15 +189,20 @@ This allows you to interpolate values from the settings or environment, and
 conditionally switch processes on or off.  The template context for each
 configuration file contains the following variables:
 
-    :PROJECT_DIR:    the top-level directory of your project (i.e. the
-                     directory containing your manage.py script).
+    :PROJECT_DIR:          the top-level directory of your project (i.e. the
+                           directory containing your manage.py script).
 
-    :APP_DIR:        for app-provided config files, the top-level directory
-                     containing the application code.
+    :APP_DIR:              for app-provided config files, the top-level
+                           directory containing the application code.
 
-    :settings:       the Django settings module, as seen by your code.
+    :PYTHON:               full path to the current python interpreter.
+
+    :SUPERVISOR_OPTIONS:   the command-line options passed to manage.py. 
+ 
+    :settings:             the Django settings module, as seen by your code.
+
+    :environ:              the os.environ dict, as seen by your code.
 
-    :environ:        the os.environ dict, as seen by your code.
 
 
 Defaults, Overrides and Excludes
@@ -236,19 +241,26 @@ Here's an example config file that shows them all in action::
     exclude=true
 
 
-Autorestart
-~~~~~~~~~~~
+
+Autoreload
+~~~~~~~~~~
 
 When running in debug mode, django-supervisor automatically defines a process
-named "autorestart".  This is very similar to the auto-reloading feature of
+named "autoreload".  This is very similar to the auto-reloading feature of
 the Django development server, but works across all configured processes.
 For example, this will let you automatically restart both the dev server and
 celeryd whenever your code changes.
 
-To switch off the autorestart process, just exclude it in your project config
-file like so::
+To prevent an individual program from being auto-reloaded, set its "autoreload"
+option to false::
 
-    [program:autorestart]
+    [program:non-python-related]
+    autoreload=false
+
+To switch off the autoreload process entirely, you can pass the --noreload 
+option to supervisor or just exclude it in your project config file like so::
+
+    [program:autoreload]
     exclude=true
 
 
@@ -262,8 +274,8 @@ first to make sure it all fits together the way I want...
 """
 
 __ver_major__ = 0
-__ver_minor__ = 1
-__ver_patch__ = 1
+__ver_minor__ = 2
+__ver_patch__ = 0
 __ver_sub__ = ""
 __version__ = "%d.%d.%d%s" % (__ver_major__,__ver_minor__,__ver_patch__,__ver_sub__)
 
